//------------------------------------------------------------------
#property copyright "www.forex-station.com"
#property link      "www.forex-station.com"
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers    4
#property indicator_color1     clrLimeGreen
#property indicator_color2     clrOrangeRed
#property indicator_color3     clrOrangeRed
#property indicator_color4     clrSilver
#property indicator_width1     2
#property indicator_width2     2
#property indicator_width3     2
#property indicator_style4     STYLE_DOT
#property indicator_level1     0
#property indicator_levelcolor clrDarkGray
//#property strict

//
//
//
//
//

enum enColorOn
{
   cc_onSlope,   // Change color on slope change
   cc_onZero,    // Change color on zero cross
   cc_onSign     // Change color on signal cross
};

input int                FastEmaPeriod        = 12;
input int                SlowEmaPeriod        = 26;
input int                SignalEmaPeriod      = 9;
input bool               AdjustSignal         = true;
input ENUM_APPLIED_PRICE Price                = PRICE_CLOSE;
input int                VolatilityPeriod     = 5;
input int                VolatilitySmooth     = 10;
input enColorOn          ColorOn              = cc_onSlope;       // Color change on :
input bool               arrowsVisible        = false;              // Arrows visible true/false?
input string             arrowsIdentifier     = "macd Arrows1";     // Unique ID for arrows
input double             arrowsUpperGap       = 0.5;                // Upper arrow gap
input double             arrowsLowerGap       = 0.5;                // Lower arrow gap
input color              arrowsUpColor        = clrLimeGreen;       // Up arrow color
input color              arrowsDnColor        = clrOrange;          // Down arrow color
input int                arrowsUpCode         = 159;                // Up arrow code
input int                arrowsDnCode         = 159;                // Down arrow code
input int                arrowsUpSize         = 2;                  // Up arrow size
input int                arrowsDnSize         = 2;                  // Down arrow size

double macd[],macdda[],macddb[],sign[],price[],sum[],trend[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int OnInit()
{
   IndicatorBuffers(7);
   SetIndexBuffer(0,macd,  INDICATOR_DATA); SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(1,macdda,INDICATOR_DATA); SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(2,macddb,INDICATOR_DATA); SetIndexStyle(2,DRAW_LINE);
   SetIndexBuffer(3,sign,  INDICATOR_DATA); SetIndexStyle(3,DRAW_LINE);
   SetIndexBuffer(4,price);
   SetIndexBuffer(5,sum);
   SetIndexBuffer(6,trend);
      
   IndicatorSetString(INDICATOR_SHORTNAME,"Volatility adjusted macd ("+(string)FastEmaPeriod+","+(string)SlowEmaPeriod+",,"+(string)SignalEmaPeriod+","+(string)VolatilityPeriod+","+(string)VolatilitySmooth+")");
return(INIT_SUCCEEDED);
}
void OnDeinit(const int reason)
{ 
    string lookFor       = arrowsIdentifier+":";
    int    lookForLength = StringLen(lookFor);
    for (int i=ObjectsTotal()-1; i>=0; i--)
    {
      string objectName = ObjectName(i);
      if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
    }
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double work[][3];
#define _emaf 0
#define _emas 1
#define _emar 2

int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
   if (ArrayRange(work,0)!=rates_total) ArrayResize(work,rates_total);
   int r,i=fmin(rates_total-prev_calculated+1,rates_total-1);
   if (trend[i]==-1) iCleanPoint(i,macdda,macddb);
   for(r=rates_total-i-1; i >= 0; i--,r++)
   {
      price[i]   =  iMA(NULL,0,1,0,MODE_SMA,Price,i);
      sum[i]     =  0;      for (int k = 0; k<VolatilityPeriod && (i+k)<rates_total; k++) sum[i] += fabs(price[i+k]-price[i+k+1]);
      double avg =  sum[i]; for (    k = 1; k<VolatilitySmooth && (i+k)<rates_total; k++) avg    += sum[i+k];
             avg /= VolatilitySmooth;
         
         //
         //
         //
         //
         //
        
         double fastPer = FastEmaPeriod;
         double slowPer = SlowEmaPeriod;
         double signPer = SignalEmaPeriod;
         if (avg!=0 && sum[i]!=0)
         {
                              fastPer = fmax(FastEmaPeriod  /(sum[i]/avg),FastEmaPeriod  /2.0);
                              slowPer = fmax(SlowEmaPeriod  /(sum[i]/avg),FastEmaPeriod  /2.0);
            if (AdjustSignal) signPer = fmax(SignalEmaPeriod/(sum[i]/avg),SignalEmaPeriod/2.0);
         }
         
            double alphaf = 2.0/(1.0+fastPer);
            double alphas = 2.0/(1.0+slowPer);
            double alphar = 2.0/(1.0+signPer); 
            work[r][_emaf] = work[r-1][_emaf]+alphaf*(price[i]                     -work[r-1][_emaf]);
            work[r][_emas] = work[r-1][_emas]+alphas*(price[i]                     -work[r-1][_emas]);
            work[r][_emar] = work[r-1][_emar]+alphar*(work[r][_emaf]-work[r][_emas]-work[r-1][_emar]);
            macd[i] = work[r][_emaf]-work[r][_emas];
            if (SignalEmaPeriod>1){ sign[i] = work[r][_emar];}  
            
            switch(ColorOn)
            {
               case cc_onSign: if (i<rates_total-1)  trend[i] = (macd[i]>sign[i])   ? 1 : (macd[i]<sign[i])   ? -1 : trend[i+1]; break;
               case cc_onZero: if (i<rates_total-1)  trend[i] = (macd[i]>0)         ? 1 : (macd[i]<0)         ? -1 : trend[i+1]; break;
               default :       if (i<rates_total-1)  trend[i] = (macd[i]>macd[i+1]) ? 1 : (macd[i]<macd[i+1]) ? -1 : trend[i+1];
            } 
            macdda[i] = macddb[i] = EMPTY_VALUE; if(trend[i] == -1) iPlotPoint(i,macdda,macddb,macd);
            
            //
            //
            //
            //
            //
            
            if (arrowsVisible)
            {
               string lookFor = arrowsIdentifier+":"+(string)time[i]; ObjectDelete(lookFor);            
               if (i<(rates_total-1) && trend[i] != trend[i+1])
               {
                 if (trend[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,arrowsUpSize,false);
                 if (trend[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode,arrowsDnSize, true);
               }
             }         
      }
return(rates_total);
}

//
//
//
//
//

void iCleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}
void iPlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i]; first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] = from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                          second[i] = EMPTY_VALUE; }
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void drawArrow(int i,color theColor,int theCode, int theSize, bool up)
{
   string name = arrowsIdentifier+":"+(string)Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //

      //datetime time = Time[i]; if (arrowsOnNewest) time += _Period*60-1;      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_WIDTH,    theSize);
         ObjectSet(name,OBJPROP_COLOR,    theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}
